
#ifndef _ORDEREDLIST_H
#define _ORDEREDLIST_H

#include <WProgram.h>

// the definition of the queue class.
template<typename T>
class OrderedList 
{
  public:
    // constructor
    OrderedList (int (*compareElements)( T*, T* ), void (*destroyElement)( T*)) ;
    // destructor
    ~OrderedList ();
    void addElement ( T* element );

    T* getPosition (int pos);
	void removeElement (byte pos);
    bool isEmpty () const;
    int count () const;
	void setPrinter (Print & p) ;
	
	private:
	void blink () const;
    static const int ledPin = 13;
	
    // the structure of each node in the list.
    typedef struct _node 
	{
      T* item;
      struct _node* nextNode;
    } tNode, *pNode;

	int (*compareElements)( T*, T*);
	void (*destroyElement)( T*);
    Print * printer; 
    int size;
    pNode head;
    pNode tail;
	pNode nodeShortcut;
	int posShortcut;
};

template<typename T>
OrderedList<T>::OrderedList (int (*compareElements)( T*, T* ), void (*destroyElement)( T*))
{
	this->compareElements = compareElements;
	this->destroyElement = destroyElement;
	size = 0;
	head = NULL;
	printer = NULL;
	nodeShortcut = NULL;
	posShortcut = 0;
}

template<typename T>
OrderedList<T>::~OrderedList () 
{
/*  for (link t = head; t != NULL; head = t) {
    t = head->next; free (head);
  } */
  size = 0;       // set the size of queue to zero.
  tail = NULL;    // set the tail of the list to point nowhere.
  printer = NULL; // set the printer of queue to point nowhere.
}

template<typename T>
void OrderedList<T>::addElement (T* element) 
{
	pNode new_node = (pNode) malloc (sizeof (tNode));
	new_node->item = element;
//	if (printer) printer->print ("Cmp:");
//	if (printer) printer->println (compareElements(element, head->item));
	pNode aux_node = head;
	pNode previous_node = NULL;
	while (aux_node != NULL && compareElements(element, aux_node->item)>0)
	{
		previous_node = aux_node;
		aux_node = aux_node->nextNode;
	}
	
	if (previous_node == NULL)
	{
		head = new_node;
		new_node->nextNode = aux_node;
	}
	else
	{
		previous_node->nextNode = new_node;
		new_node->nextNode = aux_node;
	}
	nodeShortcut = head;
	posShortcut = 1;
	size++;
}

template<typename T>
T* OrderedList<T>::getPosition (int pos)
{
	if (isEmpty () || pos>size) return NULL;
	int steps;	
	if (pos<posShortcut)
	{
		nodeShortcut = head;
		steps = pos-1;
	}
	else 
	{
		steps = pos-posShortcut;
	}	
	for (int i=0; i<steps; i++)
		nodeShortcut = nodeShortcut->nextNode;	

	posShortcut = pos;
	return nodeShortcut->item;
}

template<typename T>
void OrderedList<T>::removeElement (byte pos)
{
	if (isEmpty () || pos> size) return;
	pNode remove_node = head;
	pNode prevoius_remove_node = NULL;
	
	for(byte i=1; i<pos; i++)
	{
		prevoius_remove_node = remove_node;
		remove_node = remove_node->nextNode;
	}

	destroyElement(remove_node->item);
	
	if (prevoius_remove_node != NULL)
		prevoius_remove_node->nextNode = remove_node->nextNode;
	else head = remove_node->nextNode;
	free(remove_node);
	size--;
	nodeShortcut = head;
	posShortcut = 1;
}


// check if the queue is empty.
template<typename T>
bool OrderedList<T>::isEmpty () const 
{
	return head == NULL;
}

// get the number of items in the queue.
template<typename T>
int OrderedList<T>::count () const 
{
	return size;
}

// set the printer of the queue.
template<typename T>
void OrderedList<T>::setPrinter (Print & p) 
{
  printer = &p;
}

// led blinking method in case of error.
template<typename T>
void OrderedList<T>::blink () const {
  // set led pin as output.
  pinMode (ledPin, OUTPUT);

  // continue looping until hardware reset.
  while (true) {
    digitalWrite (ledPin, HIGH); // sets the LED on.
    delay (250);                 // pauses 1/4 of second.
    digitalWrite (ledPin, LOW);  // sets the LED off.
    delay (250);                 // pauses 1/4 of second.
  }
  // solution selected due to lack of exit() and assert().
}


#endif
